#include <string>
#if (defined (WIN32))
    #include <Windows.h>
#endif
#include <Registration.h>
#include <json.hpp>
#include <zmq.hpp>
#include "zhelpers.hpp"

using namespace mdd;
class Server {
private:
    zmq::context_t context;
    Registration regi = Registration();

    json msg_header() {
        json jmsg;
        jmsg["serverID"] = id;
        jmsg["msgNr"] = counter;
        return jmsg;
    }

    void send_alive() {
        std::string msg = msg_header().dump();

        //  Send reply back to client
        zmq::message_t reply(msg.size());
        memcpy(reply.data(), msg.c_str(), msg.size());
        reply_socket.send(reply);
    }

    void get_request(json request) {

        json jans = msg_header();
        if (request.contains("state")) {
            if (request["state"].get<std::string>() == "all")
            {
                
            }
        }
        if (request.contains("module")) {
            if (request["module"].get<std::string>() == "all")
            {
                jans["module"] = regi.getModules();
            }
        }
        if (request.contains("process")) {
            if (request["process"].get<std::string>() == "all")
            {
                jans["process"] = regi.getModules();
            }
        }

        std::string msg = jans.dump();

        //  Send reply back to client
        zmq::message_t reply(msg.size());
        memcpy(reply.data(), msg.c_str(), msg.size());
        reply_socket.send(reply);
    }

    void try_request(json request) {
        json jmodule;
        jmodule["type"] = "module";
        json jmsg = msg_header();
        if (request.contains("add")) {
            jmsg["id"] = request["id"];
            jmodule["id"] = request["add"];
            jmsg["add"] = jmodule;
        }

        std::string smsg = jmsg.dump();
        std::string stopic = "CHANGE";
        zmq::message_t topic(stopic.size());
        zmq::message_t msg(smsg.size());
        memcpy(topic.data(), stopic.c_str(), stopic.size());
        memcpy(msg.data(), smsg.c_str(), smsg.size());

        Sleep(500);
        try {
            std::cout << "Send: " << stopic << ": " << smsg << std::endl;
            s_sendmore(publisher_socket, stopic);
            s_send(publisher_socket, smsg);
            //publisher_socket.send(topic, ZMQ_SNDMORE);
            //publisher_socket.send(msg);
        }
        catch (zmq::error_t& e) {
            std::cout << e.what() << std::endl;
        }
        std::cout << "Test" << std::endl;
        //msg.rebuild(3);
        //topic.rebuild(4);

        ++counter;
    }

public:
    zmq::socket_t reply_socket;
    zmq::socket_t publisher_socket;
    
    uint32_t id = 0;
    size_t counter = 0;

    Server()
        : context(1)
        , reply_socket(context, zmq::socket_type::rep)
        , publisher_socket(context, zmq::socket_type::pub)
    {
        id = reinterpret_cast<uint32_t>(this);
        reply_socket.bind("tcp://*:5555");
        publisher_socket.bind("tcp://*:5556");
    }

    void handle_request() {
        zmq::message_t request;
        //  Wait for next request from client
        try
        {
            reply_socket.recv(&request);
            std::cout << request.to_string() << std::endl;
        }
        catch (const std::exception&)
        {
            std::cout << "[Server] recived wrong msg!" << std::endl;
        }
        json msg;
        try
        {
            msg = json::parse(request.to_string());
        }
        catch (const std::exception&)
        {
            std::cout << "[Server] recived wrong msg: " << request.to_string() << std::endl;
        }
        if (msg.contains("try"))
        {
            try_request(msg["try"]);
            send_alive();
        }
        if (msg.contains("get"))
        {
            get_request(msg["get"]);
        }
        //

        //  Do some 'work'
        Sleep(1000);
       
    }

};


int main(int argc, char* argv[]) {
    Server server;

    while (true) {
        server.handle_request();
        std::cout << "Handeld" << std::endl;
    }
    return 0;
}